@@ -540,17 +540,21 @@ static gboolean ignore_mount_entry(const struct mount_entry *me)
 	static const char ignored[][12] = {
 		"autofs",
 		"binfmt_misc",
+		"ctfs",
 		"devfs",
 		"devpts",
+		"fusectl",
 		"linprocfs",
 		"mfs",
 		"mntfs",
 		"mqueue",
-		"nsfd",
 		"none",
+		"nsfd",
+		"objfs",
 		"openpromfs",
 		"proc",
 		"procfs",
+		"securityfs",
 		"supermount",
 		"sysfs",
 		"tmpfs",
